﻿/*
Расширить движок снегопата можно скриптом на AngelScript.
Все файлы с расширением *.as из папок 'core\engine' и 'mods' загружаются в единый модуль и компилируются в
движок снегопата в момент его старта.

Если вам не нужно глубоко менять поведение снегопата, а достаточно добавить функционал в виде аддина,
то в снегопате есть простая возможность сделать встроенный аддин на AngelScript.
Для этого вам нужно создать класс наследник от BuiltinAddin, создать глобальный объект
этого класса, и в конструкторе вызвать конструктор базового класса.
Аддин будет показан в ветке "Служебные аддины"
*/

// Это глобальный объект нового аддина. Имя давайте уникальное, так как всё происходит в едином пространстве с движком снегопата
SimpleBuiltinAddin oneSimpleBuiltinAddin;

// А это класс для аддина. Название давайте уникальное, наследуйтесь от BuiltinAddin
class SimpleBuiltinAddin : BuiltinAddin {
	// Это конструктор - название совпадает с названием класса
    SimpleBuiltinAddin()
    {
		// Вызываем базовый конструктор - передаём уникальное имя и отображаемое имя, аналог uname и dname для скриптов
		// последним параметром предаем order - аддины подключаются в порядке возврастания order.
		// Применяется, если ваш аддин зависит от другого встроенного аддина и должен грузится после него
        super("simplebuiltinaddin", "Мой простой встроенный в снегопат аддин на AngelScript", 0);
    }
	// Метод возвращает список макросов аддина
	array<string>&& macroses() {
		return array<string> = {"Макрос 1", "Макрос 2"};
	}
	// Метод обрабатывает вызов макроса
	Variant invokeMacros(const string& macros) {
		if (macros == "Макрос 1")
			Message("Вызван макрос 1");
		else if (macros == "Макрос 2")
			MsgBox("Вызван макрос 2");
		return Variant();
	}
	
    // получить массив имен команд меню. Пока снегопатом не используется
    array<string>&& menu() {
		return array<string> = {"Меню 1", "Меню 2"};
	}
    // Выполнить команду меню
    void invokeMenu(uint idx) {
	}
	
	/* Это функция для получения в скриптах объекта аддина. При реализации этого метода скрипты смогут подключатся к вашему аддину
	   и вызывать методы этого объекта, например
	
	//addin: simplebuiltinaddin sba
	...
	Message(sba.metod1());
	
	 в данном случае создаём простой объект
	*/
    IUnknown&& object()
    {
		// Если объект еще не создали - создадим
        if (obj is null)
            &&obj = createDispatchFromAS(&&MySimpleBuiltinAddinObject());
        return obj;
    }
    private IDispatch&& obj;
};

// Это класс, реализующий скриптовый объект для аддина
class MySimpleBuiltinAddinObject {
	string method1() {
		return "Hello from me!";
	}
}
